// kk_inc_dsg     --  22/01/2016
// Include script for all disguise armband functions.
// Created By: Kitty Kitty - PM comments to me on http://www.lucasforums.com/

/* The real work happens here.  We gather values from parameters, tables, or
  loading from stored globals.  We then determine whether to apply a head,
  body, both, or to do a base appearance restoration to the user.

Game Globals: KK_DsgBandVars, KK_DsgBandVars2, KK_DsgBandVars3 (Locations):
  Locations are: vector (float x, float y, float z), facing (float).  We need
  3 ints for the PC, and 1 for each NPC (12 total) ranging from 0 to 508+
  (depends on custom content). The game globals can't store integers over
  -127/+128 (signed int) so for the PC, we use the 3 vector floats from
  KK_DsgBandVars and each NPC gets their own single float assigned if/when
  they use the item. Float values are more than enough for our needs, and are
  easily converted to int or string where needed.                           */

// Pseudo-Constants --We can't define consts except by editing nnwscript.nss,
// which we usually want to avoid.  These work though, as long as we don't
// accidentally change them.  Note that you can not use these in all places
// you could a true constant. The compiler will alert you if this happens.
object MAIN_CHAR  = GetFirstPC(); // Main PC (user-created character)
object USER_CHAR  = OBJECT_SELF;  // User of armband as object.

// ATTENTION: Set the value below to the LAST row in the appearance.2da in 
// your override folder.  This is an added safety check to ensure we do not
// attempt to set an out of range appearance, which will do bad things.
int MAX_APPEARANCE = 508;         

// Struct Template to hold all our data for easy manipulation.
struct GlobNums { // Sets up who gets which float(s) from the globals.
  int PCBase;     // Global 1, Vector 1 - PC Base Appearance
  int PCBody;     // Global 1, Vector 2 - PC Last Used Body
  int PCHead;     // Global 1, Vector 3 - PC Last Used Head
  int T3M4Head;   // Global 1, Facing   - T3-M3 Last Used Head
  int HK47Head;   // Global 2, Vector 1 - HK-74 Last Used Head
  int BastHead;   // Global 2, Vector 2 - Bastila Last Used Head 
  int CandHead;   // Global 2, Vector 3 - Canderous Last Used Head
  int CartHead;   // Global 2, Facing   - Carth Last Used Head
  int JoleHead;   // Global 3, Vector 1 - Jolee Last Used Head 
  int JuhaHead;   // Global 3, Vector 2 - Juhani Last Used Head
  int ZaalHead;   // Global 3, Vector 3 - Zaalbar Last Used Head
  int MissHead;   // Global 3, Facing   - Mission Last Used Head
 };

// Global vars altered by script functions at run-time.
// Var to track who used the armband.  Set to 100 for a simple 'unused' check.
// Since once it is in use, -1 is for PC, while NPCs range 0 through 8
int BandUser =  100;  

// Function Declarations - See each function for comments.
struct GlobNums KK_GetData();
struct GlobNums KK_CheckData(struct GlobNums gData);
void KK_SaveData(struct GlobNums gData);
int KK_GetNPC();
int KK_GetBaseApp();
void KK_SetData(int iBody, int iHead);
int KK_ModelTest(int iVal);
void KK_ApplyDisguise(int iBase, int iBody, int iHead);
void KK_SetDummyApp(object dChar, int iBase, int iBody, int iHead);
int KK_GetStat(int sVar = 2);
void KK_FullDisguise(object dChar, int iBase, int iBody, int iHead);
void KK_RemoveDisguise();
void KK_DisguiseBody(int iBody);
void KK_DisguiseHead(object dChar);

// This uses holding vars to read the stored globals one at a time and pull
// out the floats, converting them to integers so we can work with them.
// We add 0.5 before conversion due to precision errors in the game's
// GetPositionFromLocation() and GetFacingFromLocation() functions.
struct GlobNums KK_GetData() {
  struct GlobNums ValToRet;       // struct instance to return
  location Glob;                  // holds one location global at a time
  vector gVec;                    // holds each set of 3 vectors in turn
  float gFloat;                   // holds the remaining positions in turn
  
  // Set 1: "KK_DsgBandVars": PC Base, Body, Head and T3-M4's head
  Glob = GetGlobalLocation("KK_DsgBandVars");   // Load first global
  gVec = GetPositionFromLocation(Glob);         // pull out first 3 floats
  gFloat = GetFacingFromLocation(Glob);         // pull the facing
  ValToRet.PCBase   = FloatToInt(gVec.x + 0.5); // Set PC Base App to f.x
  ValToRet.PCBody   = FloatToInt(gVec.y + 0.5); // Set PC New Body to f.y
  ValToRet.PCHead   = FloatToInt(gVec.z + 0.5); // Set PC New Head to f.z
  ValToRet.T3M4Head = FloatToInt(gFloat + 0.5); // Set T3-M4 Head to f4
  // Set 2: "KK_DsgBandVars2": HK-47, Bastila, Canderous, and Carth's heads
  Glob = GetGlobalLocation("KK_DsgBandVars2");  // Load second global
  gVec = GetPositionFromLocation(Glob);         // pull out second 3 floats
  gFloat = GetFacingFromLocation(Glob);         // ...and the orientation
  ValToRet.HK47Head = FloatToInt(gVec.x + 0.5); // HK-74 New Head = fx
  ValToRet.BastHead = FloatToInt(gVec.y + 0.5); // Bastila New Head = fy
  ValToRet.CandHead = FloatToInt(gVec.z + 0.5); // Canderous New Head = fz
  ValToRet.CartHead = FloatToInt(gFloat + 0.5); // Carth New Head = f4
  // Set 3: "KK_DsgBandVars3": Jolee, Juhani, Zaalbar, and Mission's heads
  Glob = GetGlobalLocation("KK_DsgBandVars3");  // Load third global
  gVec = GetPositionFromLocation(Glob);         // pull third 3 floats
  gFloat = GetFacingFromLocation(Glob);         // ...and the orientation
  ValToRet.JoleHead = FloatToInt(gVec.x + 0.5); // Jolee New Head = fx
  ValToRet.JuhaHead = FloatToInt(gVec.y + 0.5); // Juhani New Head = fy
  ValToRet.ZaalHead = FloatToInt(gVec.z + 0.5); // Zaalbar New Head = fz
  ValToRet.MissHead = FloatToInt(gFloat + 0.5); // Mission New Head = f4

  return ValToRet; // Returns the whole struct, loaded with fresh data
}

// Pass a struct to this function to set any zero (0) or one (1) value
// variables to something useful.  Mostly this is to initialize variables
// which have been created, but not yet used before we store the globals.
// If you have altered any NPCs base appearance, change the numbers found
// below to reflect the new row they use in appearance.2da
struct GlobNums KK_CheckData(struct GlobNums gData) {
  int tApp; // Temp Var for current appearance type.
  tApp = GetAppearanceType(MAIN_CHAR);   // Gets current PC Appearance

  // If any value loads in as less than 2 (usually unused 0 values)...
  if (gData.PCBase < 2)   {gData.PCBase = tApp;}  // set to PC appearance
  if (gData.PCBody < 2)   {gData.PCBody = tApp;}  // set to PC appearance
  if (gData.PCHead < 2)   {gData.PCHead = tApp;}  // set to PC appearance
  if (gData.T3M4Head < 2) {gData.T3M4Head = 2;}   // set to T3-M4 base (2)
  if (gData.HK47Head < 2) {gData.HK47Head = 3;}   // set to HK-47 base (3)
  if (gData.BastHead < 2) {gData.BastHead = 4;}   // set to Bastila base (4)
  if (gData.CandHead < 2) {gData.CandHead = 5;}   // set to Canderous base (5)
  if (gData.CartHead < 2) {gData.CartHead = 6;}   // set to Carth base (6)
  if (gData.JoleHead < 2) {gData.JoleHead = 7;}   // set to Jolee base (7)
  if (gData.JuhaHead < 2) {gData.JuhaHead = 8;}   // set to Juhani base (8)
  if (gData.ZaalHead < 2) {gData.ZaalHead = 9;}   // set to Zaalbar base (9)
  if (gData.MissHead < 2) {gData.MissHead = 10;}  // set to Mission base (10)
  // ...We set it to that (N)PCs default appearance.
  KK_SaveData(gData);    // Then we store the globals
  return (gData);        // and return the altered version
}

// This function saves whatever values are passed to it back to the game-
// persistent global location variables.  Do not send data directly to
// this function, but rather pass body/head values through KK_SetData()
void KK_SaveData(struct GlobNums gData) {
  vector gVec;    // Builds a vector (float.x, float.y, float.z)
  location gLoc;  // Builds a location (vector, facing (float))

  // Group/Global 1
  gVec = Vector(IntToFloat(gData.PCBase),     // PC Base Appearance
                IntToFloat(gData.PCBody),     // PC Body
                IntToFloat(gData.PCHead));    // PC Head
  gLoc = Location(gVec,                       // Vector we just built
                IntToFloat(gData.T3M4Head));  // T3-M4 Head
  SetGlobalLocation("KK_DsgBandVars",gLoc);   // Store the global
  // Group/Global 2
  gVec = Vector(IntToFloat(gData.HK47Head),   // HK-47 Head
                IntToFloat(gData.BastHead),   // Bastila Head
                IntToFloat(gData.CandHead));  // Canderous Head
  gLoc = Location(gVec,                       // Vector we just built
                IntToFloat(gData.CartHead));  // Carth Head
  SetGlobalLocation("KK_DsgBandVars2",gLoc);  // Store global 2
  // Group/Global 3
  gVec = Vector(IntToFloat(gData.JoleHead),   // Jolee Head
                IntToFloat(gData.JuhaHead),   // Juhani Head
                IntToFloat(gData.ZaalHead));  // Zaalbar Head
  gLoc = Location(gVec,                       // Vector we just built
                IntToFloat(gData.MissHead));  // Mission Head
  SetGlobalLocation("KK_DsgBandVars3",gLoc);  // Store Value to global 3
}

// This function returns an integer we can use to easily tell who's using the
// armband for storing/reading data or switching control back to user.
int KK_GetNPC() {
  int tVar;     // so we have something to put our return value in.
  string sTemp; // to hold our object tag string.

  sTemp = GetStringLowerCase(GetTag(OBJECT_SELF));  // Get user's "tag" 
  // For custom NPCs, add 'else if' lines with the NPCs 'tag' in lower-case.
  // Set tVar equal to the row they use now in appearance.2da. If you change
  // an NPCs default appearance, change their number to equal the row they
  // now use. "BandUser" is an int value matching NPC "slots", so custom NPCs
  // should be able to use the same CONSTANT for whomever you replace.
  if      (sTemp == "")   			{tVar = NPC_PLAYER;}     //-1 (PC has no tag)
  else if (sTemp == "bastila")  {tVar = NPC_BASTILA;}    // 0
  else if (sTemp == "cand")     {tVar = NPC_CANDEROUS;}  // 1
  else if (sTemp == "carth")    {tVar = NPC_CARTH;}      // 2
  else if (sTemp == "hk47")     {tVar = NPC_HK_47;}      // 3
  else if (sTemp == "jolee")    {tVar = NPC_JOLEE;}      // 4
  else if (sTemp == "juhani")   {tVar = NPC_JUHANI;}     // 5
  else if (sTemp == "mission")  {tVar = NPC_MISSION;}    // 6
  else if (sTemp == "t3m4")     {tVar = NPC_T3_M4;}      // 7
  else if (sTemp == "zaalbar")  {tVar = NPC_ZAALBAR;}    // 8
  // In case anything bizarre happens, default to the PC and send an error
  // message in feedback screen, since this should really never happen.
  else                          {tVar = NPC_PLAYER;      //-1
    // DEBUG MESSAGES - Since this result should NOT happen, I'm putting in a
    // Debug message that's "always on". Please report if you see it in game.
    SendMessageToPC(OBJECT_SELF,"SOMETHING IS WRONG - USING DEFAULT VALUE");
    SendMessageToPC(OBJECT_SELF,"Tag read as: " + sTemp);
    // END DEBUG
  } 
  return tVar;  // Sends our constant value back to BandUser.
}

// This function returns an effective "Base Appearance" for a given user.
int KK_GetBaseApp() {
  int tVar;                                 // to put our return value in.

  switch (BandUser) {                       // Who's using armband?
    // If you change any NPC's default appearance, you should also set their numbers 
    // here to match the row they are now using and re-compile all scripts.
    case NPC_PLAYER:    tVar = KK_GetStat(0); break;  // PC's Base Appearance
    case NPC_BASTILA:   tVar = 4;   break;  // Bastila Default Body
    case NPC_CANDEROUS: tVar = 5;   break;  // Canderous Default Body
    case NPC_CARTH:     tVar = 6;   break;  // Carth  Default Body
    case NPC_HK_47:     tVar = 3;   break;  // HK-47 Default Body
    case NPC_JOLEE:     tVar = 7;   break;  // Jolee Default Body
    case NPC_JUHANI:    tVar = 8;   break;  // Juhani Default Body
    case NPC_MISSION:   tVar = 10;  break;  // Mission Default Body
    case NPC_T3_M4:     tVar = 2;   break;  // T3-M4 Default Body
    case NPC_ZAALBAR:   tVar = 9;   break;  // Zaalbar Default Body
    default:            tVar = KK_GetStat(0); break;  // PC base with error msg.
      SendMessageToPC(OBJECT_SELF, "Error: BandUser out of" +
      "range : " + IntToString(BandUser));  break;
  }
 return tVar;
}

// Call this to set new values for body (PC only) or head.  It does some
// checks to prevent bad data before auto-calling the save function.
void KK_SetData(int iBody, int iHead) {
  int tVar;                       // Used to check validity of head model.
  struct GlobNums globData;       // We need a struct to manipulate.

  globData = KK_GetData();        // fill struct with data from globals.
  if (iBody >= 2) {               // we don't want any zero values saved.
    if (BandUser == NPC_PLAYER) { // only the PC gets a body value stored.
      globData.PCBody = iBody;    // update struct with new data.
    }
  }
  // Test for valid head model. If invalid, do not update global.
  if (KK_ModelTest(iHead) == TRUE) {
    switch (BandUser) {   // Find matching user to set data to.
      case NPC_PLAYER:    globData.PCHead   = iHead;  break;  // -1
      case NPC_BASTILA:   globData.BastHead = iHead;  break;  // 0
      case NPC_CANDEROUS: globData.CandHead = iHead;  break;  // 1
      case NPC_CARTH:     globData.CartHead = iHead;  break;  // 2
      case NPC_HK_47:     globData.HK47Head = iHead;  break;  // 3
      case NPC_JOLEE:     globData.JoleHead = iHead;  break;  // 4
      case NPC_JUHANI:    globData.JuhaHead = iHead;  break;  // 5
      case NPC_MISSION:   globData.MissHead = iHead;  break;  // 6
      case NPC_T3_M4:     globData.T3M4Head = iHead;  break;  // 7
      case NPC_ZAALBAR:   globData.ZaalHead = iHead;  break;  // 8
      default: SendMessageToPC(OBJECT_SELF, "Error: BandUser out of" +
        "range : " + IntToString(BandUser));  break;
    }
  }
  KK_SaveData(globData);
}

// Function to test whether a head model actually exists.  If this function
// returns TRUE then both a BODY and HEAD model exist of that number, while
// FALSE returns are one-piece (BODY only) models or invalid (out of range).
int KK_ModelTest(int iVal) { // 'iVal' holds the parameter value at runtime.
  int validHead =
       iVal >   3 && iVal < 9     || iVal >   9 && iVal < 12
    || iVal >  13 && iVal < 19    || iVal == 23
    || iVal >  24 && iVal < 28    || iVal >  29 && iVal < 32
    || iVal >  32 && iVal < 35    || iVal >  36 && iVal < 39
    || iVal >  40 && iVal < 44    || iVal >  46 && iVal < 49 
    || iVal >  90 && iVal < 181   || iVal > 183 && iVal < 228 
    || iVal > 268 && iVal < 271   || iVal == 280 
    || iVal > 282 && iVal < 309   || iVal > 338 && iVal < 351 
    || iVal > 354 && iVal < 368   || iVal > 370 && iVal < 373 
    || iVal > 374 && iVal < 379   || iVal > 379 && iVal < 382 
    || iVal > 384 && iVal < 392   || iVal > 411 && iVal < 418 
    || iVal > 422 && iVal < 427   || iVal > 427 && iVal < 436 
    || iVal == 438                || iVal == 442 
    || iVal > 443 && iVal < 451   || iVal > 453 && iVal <= MAX_APPEARANCE;
    // Set MAX_APPEARANCE (top of this file) equal to the last row (line) in use 
    // in the appearance.2da file in your override folder.
  if (validHead) {
    return TRUE;
  } else {
    return FALSE;
  }
}

// Main Disguise function.  Really just a data manipulation wrapper that
// calls out to various other functions, but this is what we call from other
// scripts to finally do something useful with all this data.
void KK_ApplyDisguise(int iBase, int iBody, int iHead) {
  object DummyChar;                       // Object for our "dummy"
  location dLoc;                          // Where to put the dummy.
  dLoc = GetStartingLocation();           // Module Start Location.

  // Create a test-dummy.  Don't worry.  We'll delete her after surgery.
  DummyChar = CreateObject(OBJECT_TYPE_CREATURE, "n_commf", dLoc, FALSE);
  // Try to find a valid head for the dummy
  KK_SetDummyApp(DummyChar, iBase, iBody, iHead);

  if (iBody == iHead) {					  // Full disguise or base restoration.
    KK_FullDisguise(DummyChar, iBase, iBody, iHead);
  } else                                  // BODY *or* HEAD
  if (iBody != 0) {			  // If we need a body...
    KK_RemoveDisguise();                  // Clear residual disguises.
    KK_DisguiseBody(iBody);		  // Disguise body
    if (KK_ModelTest(iBody) == TRUE) {	  // If we have a valid head...
      KK_DisguiseHead(DummyChar);         // change our head.
    }
  } else
  if (iHead != 0) {                    	  // If we need a head...
      KK_DisguiseHead(DummyChar);         // change our head.
  }
  DestroyObject(DummyChar);               // Get rid of dummy once we're all done
}

// This rather ugly function attempts to find a valid head type to apply to
// the dummy NPC by checking the New Head, Saved Head, Default Head, and New
// Body in order. If none of these are valid head models, we just leave it be.
void KK_SetDummyApp(object dChar, int iBase, int iBody, int iHead) {
  int iStat = KK_GetStat();           // holder for head number.

  if (KK_ModelTest(iHead) == TRUE) {  // New Head is Valid?
    ApplyEffectToObject(DURATION_TYPE_PERMANENT,
      EffectDisguise(iHead), dChar);  // Disguise dummy with New Head.
  } else
  if (KK_ModelTest(iStat) == TRUE) {  // Last Head is Valid?
    ApplyEffectToObject(DURATION_TYPE_PERMANENT,
      EffectDisguise(iStat), dChar);  // Disguise dummy with Last Head.
  } else
  if (KK_ModelTest(iBody) == TRUE) {  // New Body is Valid?
    ApplyEffectToObject(DURATION_TYPE_PERMANENT,
      EffectDisguise(iBody), dChar);  // Disguise dummy with New Body.
  } else
  if (KK_ModelTest(iBase) == TRUE) {  // Base Appearance is Valid?
    ApplyEffectToObject(DURATION_TYPE_PERMANENT,
      EffectDisguise(iBase), dChar);  // Disguise dummy with Base Head.
  // If anyone sees the following message in the feedback screen, PLEASE let me know
  } else { SendMessageToPC(OBJECT_SELF, "ERROR: NO VALID HEAD FOUND!");
  }
}

// Function used to return only a single stat, rather than the entire struct
int KK_GetStat(int sVar = 2) {
  int retVal;               // Value to be returned
  struct GlobNums tGlob;    // We need the struct to get the data from

  tGlob = KK_GetData();     // Loads data into the struct
  switch (BandUser) {       // Who's data are we looking for?
    case NPC_PLAYER:        // Dealing with the PC.
      switch (sVar) {       // For PC, we need to determine which stat.
        case 0:         retVal = tGlob.PCBase;  break;	// Get PC Base
        case 1:         retVal = tGlob.PCBody;  break;  // Get PC Body
        case 2:         retVal = tGlob.PCHead;  break;  // Get PC Head
        default:        retVal = tGlob.PCHead;  // PC Head with error msg.
          SendMessageToPC(OBJECT_SELF, "Error: switchVar out of range: " +
            IntToString(sVar));  break;
      } break;
    case NPC_BASTILA:   retVal = tGlob.BastHead;  break; // Get Bastila Head
    case NPC_CANDEROUS: retVal = tGlob.CandHead;  break; // Get Canderous Head
    case NPC_CARTH:     retVal = tGlob.CartHead;  break; // Get Carth Head
    case NPC_HK_47:     retVal = tGlob.HK47Head;  break; // Get HK-74 Head
    case NPC_JOLEE:     retVal = tGlob.JoleHead;  break; // Get Jolee Head
    case NPC_JUHANI:    retVal = tGlob.JuhaHead;  break; // Get Jujani Head
    case NPC_MISSION:   retVal = tGlob.MissHead;  break; // Get Mission Head
    case NPC_T3_M4:     retVal = tGlob.T3M4Head;  break; // Get T3-M4 Head
    case NPC_ZAALBAR:   retVal = tGlob.ZaalHead;  break; // Get Zaalbar Head
    default:            retVal = tGlob.BastHead;  // Return Bast Head + Error
      SendMessageToPC(OBJECT_SELF, "Error: BandUser out of range: " +
      IntToString(BandUser));  break;
  } 
  return retVal;  // Send our value back to whatever function asked for it.
}

// This function handles full-set disguises and base restorations.
void KK_FullDisguise(object dChar, int iBase, int iBody, int iHead) {
  KK_RemoveDisguise();                    // Clear residual disguise effects.
  KK_DisguiseBody(iBody);                 // Call body disguise function.
  if (KK_ModelTest(iBody) == TRUE) {      // If head model exists...
    KK_DisguiseHead(dChar);               // Disguise Head.
  }
  if (iBase == iBody && iBase == iHead) { // Must be a base restoration.
    KK_RemoveDisguise();                  // so clear residual disguises.
  }
}

// Simple loop to get rid of active disguise effects. Mainly so we don't
// stack 20 effects when only the last one applied actually does anything.
void KK_RemoveDisguise() {
  effect eff;

  eff = GetFirstEffect(USER_CHAR);  // Find first effect on user.
  while(GetIsEffectValid(eff)) {    // so long as we find effects...
    if(GetEffectType(eff) == 
      EFFECT_TYPE_DISGUISE) {       // If effect type is disguise...
      RemoveEffect(USER_CHAR, eff); // Remove the effect.
    }
    eff = GetNextEffect(USER_CHAR); // Get next effect, repeat as needed.
  }
}

// The disguise effect with our selected body number gets applied here.
void KK_DisguiseBody(int iBody) {
  ApplyEffectToObject(DURATION_TYPE_PERMANENT,
    EffectDisguise(iBody), USER_CHAR);   // Disguise user with selected body
}

// This makes sure the dummy NPC exists, then copys its head to our (N)PC.
void KK_DisguiseHead(object dChar) {
  do {
    DuplicateHeadAppearance(USER_CHAR, dChar);  // Copy dummy's head
  } while (!GetIsObjectValid(dChar));
  DuplicateHeadAppearance(USER_CHAR, dChar);    // Copy dummy's head
}

