@ECHO off
CLS
setlocal EnableDelayedExpansion

IF "%1"=="" GOTO ErrorHandler
IF %1 NEQ +%1 GOTO ErrorHandler

:: This little batch script will automate compiling all source scripts needed for a given
:: installation of Kitty Kitty's disguise armband.  
:: Steps to compile all disguise armband scripts:
:: 1. Ensure that a valid copy of the KotOR version of nwnscript.nss exists either in the main
::    game folder or in the source script folder.
:: 2. Be sure you have a copy of nwnnsscomp.exe in the source script folder.
:: 3. Determine which of the include scripts you need, and copy it to the script source folder.
:: 4. Run the batch file, passing at least one number (the maximum count) for the highest number
::    files you wish to compile (usually 508, 511, or 514).  For example:
::
:: > compile_scripts 20
:: will compile all kk_dsg_bodyxxx kk_dsg_fullxxx and kk_dsg_headxxx scripts from 001 to 020
:: provided a source script of that number exists --by default, there are some gaps that are
:: not necessary for the item to function correctly.
::
:: you can also pass a 'start number' as a second parameter, such as:
::
:: > compile_scripts 200 150
:: will compile all the same series of scripts, but beginning at 150 and ending at 200
::
:: Note that if you DO pass a starting parameter, the batch script will skip the three
:: additional scripts: kk_disguise.nss, kk_dsg_ldr.nss and kk_dsg_restore.nss, and that
:: these scripts will ALWAYS be compiled any time you do NOT specify a starting number.
::
:: Once finished, look for the "output" folder in this source folder for your compiled scripts.

GOTO CompileStart

:ErrorHandler
	ECHO Basic Usage:
	ECHO Compile_Scripts [end number] ^<start number^>
	ECHO.
	ECHO      [end number] and ^<start number^> can be any real, whole nummber.
	ECHO      if ^<start number^> is NOT given, the value 1 will be used.
	ECHO      NOTE: The only way to compile the 3 supplimental scripts ^(kk_disguise,
	ECHO      kk_dsg_ldr, and kk_dsg_restore^) is to NOT use any second parameter.
	ECHO.
	ECHO [end number]  sets the upper limit for files to compile.
	ECHO ^<start number^> is optional, and sets the starting file number.
	ECHO Typical values for [end number] are 508 for the base game, 511 for KK_PC or 
	ECHO K1R, and 514 with both mods installed.
	ECHO.
	ECHO Example: Compile_Scripts 200 100
	ECHO The above command will begin at kk_dsg_name100 and continue to kk_dsg_name200,
	ECHO compiling every valid nss file it finds in between where 'name' represents 
	ECHO body, head, and full for each iteration.
	IF %1 NEQ +%1 (
		IF "%1" NEQ "" (
			ECHO.
			ECHO Parameters must be numbers.
		)
	)
	SET ERRCHECK=1
	GOTO CompileEnd

:CompileStart
	IF NOT EXIST output\ MD output
	ECHO. >> Compile.Log
	ECHO. >> Compile.Log
	ECHO %date% >> Compile.Log
	ECHO %time% >> Compile.Log
	ECHO. >> Compile.Log
	ECHO.
	ECHO.
	ECHO Compiling scripts....

	IF "%2"=="" (
		SET START=1
		ECHO Compiling: kk_disguise.nss
		nwnnsscomp.exe -c -g 1 --outputdir output --optimize kk_disguise.nss >> Compile.log
		ECHO Compiling: kk_dsg_ldr.nss
		nwnnsscomp.exe -c -g 1 --outputdir output --optimize kk_dsg_ldr.nss >> Compile.log
		ECHO Compiling: kk_dsg_restore.nss
		nwnnsscomp.exe -c -g 1 --outputdir output --optimize kk_dsg_restore.nss >> Compile.log
	) ELSE (SET START=%2)

	FOR /L %%A IN (%START%, 1, %1) DO (
		SET B=00%%A
		SET B=!B:~-3!
		SET C=kk_dsg_body!B!
		SET D=kk_dsg_full!B!
		SET E=kk_dsg_head!B!
		ECHO Compiling: !C!.nss
		nwnnsscomp.exe -c -g 1 --outputdir output --optimize !C!.nss >> Compile.log
		ECHO Compiling: !D!.nss
		nwnnsscomp.exe -c -g 1 --outputdir output --optimize !D!.nss >> Compile.log
		ECHO Compiling: !E!.nss
		nwnnsscomp.exe -c -g 1 --outputdir output --optimize !E!.nss >> Compile.log
	)
	ECHO.
	ECHO Results saved to Compile.Log
	
:CompileEnd
	IF "%ERRCHECK%" NEQ "1" (
		ECHO.
		ECHO Compile_Scripts Complete.
	)
